@@ -1,3 +1,5 @@
+require 'json'
+
 Shindo.tests('Excon basics') do
   with_rackup('basic.ru') do
     basic_tests
@@ -89,12 +91,25 @@ end
 Shindo.tests('Excon basics (Basic Auth Pass)') do
   with_rackup('basic_auth.ru') do
     basic_tests('http://test_user:test_password@127.0.0.1:9292')
+    user, pass, uri = ['test_user', 'test_password', 'http://127.0.0.1:9292'].map(&:freeze)
+
     tests('with frozen args').returns(200) do
-      user, pass, uri = ['test_user', 'test_password', 'http://127.0.0.1:9292'].map(&:freeze)
-      connection = Excon.new(uri, :user => user, :password => pass )
-      response = connection.request(:method => :get, :path => '/content-length/100')
+      connection = Excon.new(uri, :method => :get, :password => pass, :path => '/content-length/100', :user => user)
+      response = connection.request
       response.status
-    end  
+    end
+
+    tests('with user/pass on request').returns(200) do
+      connection = Excon.new(uri, :method => :get, :path => '/content-length/100')
+      response = connection.request(:user => user, :password => pass)
+      response.status
+    end
+
+    tests('with user/pass on connection and request').returns(200) do
+      connection = Excon.new(uri, :method => :get, :password => 'incorrect_password', :path => '/content-length/100', :user => 'incorrect_user')
+      response = connection.request(user: user, password: pass)
+      response.status
+    end
   end
 end
 
@@ -187,11 +202,13 @@ Shindo.tests('Excon basics (ssl file)',['focus']) do
       connection.request(:method => :get, :path => '/content-length/100')
     end
 
-    basic_tests('https://127.0.0.1:8443',
-                :client_key => File.join(File.dirname(__FILE__), 'data', 'excon.cert.key'),
-                :client_cert => File.join(File.dirname(__FILE__), 'data', 'excon.cert.crt')
-               )
+    cert_key_path = File.join(File.dirname(__FILE__), 'data', 'excon.cert.key')
+    cert_crt_path = File.join(File.dirname(__FILE__), 'data', 'excon.cert.crt')
+    basic_tests('https://127.0.0.1:8443', client_key: cert_key_path, client_cert: cert_crt_path)
 
+    cert_key_data = File.read cert_key_path
+    cert_crt_data = File.read cert_crt_path
+    basic_tests('https://127.0.0.1:8443', client_key_data: cert_key_data, client_cert_data: cert_crt_data)
   end
 end
 
@@ -246,6 +263,19 @@ Shindo.tests('Excon basics (Unix socket)') do
         response[:status]
       end
     end
+
+    tests('http Host header is empty') do
+      tests('GET /headers').returns("") do
+        connection = Excon::Connection.new({
+          :socket           => file_name,
+          :nonblock         => false,
+          :scheme           => 'unix',
+          :ssl_verify_peer  => false
+        })
+        response = connection.request(:method => :get, :path => '/headers')
+        JSON.parse(response.body)['HTTP_HOST']
+      end
+    end
   end
 end
 
@@ -285,7 +315,7 @@ Shindo.tests('Excon basics (reusable local port)') do
     end
   end
 
-  with_rackup('basic.ru') do
+  with_rackup('basic.ru', '0.0.0.0') do
     connection = Excon.new("http://127.0.0.1:9292/echo",
                            :reuseaddr => true, # enable address and port reuse
                            :persistent => true # keep the socket open
